@@ -1,15 +1,13 @@
 __docformat__ = 'google'
 
 import icepool
-from icepool.collections import Counts
-from icepool.typing import Outcome
+from icepool.collection.counts import Counts
 
-import itertools
 import math
 from collections import defaultdict
 
 from icepool.typing import T
-from typing import Any, Iterable, Mapping, MutableMapping, Sequence
+from typing import Any, Mapping, MutableMapping, Sequence, cast, overload
 
 
 def itemize(keys: Mapping[Any, int] | Sequence,
@@ -31,7 +29,7 @@ def itemize(keys: Mapping[Any, int] | Sequence,
         raise TypeError('Argument appears not to be a mapping or sequence.')
 
     if isinstance(times, int):
-        times = (times,) * len(keys)
+        times = (times, ) * len(keys)
     else:
         if len(times) != len(keys):
             raise ValueError(
@@ -55,8 +53,7 @@ def expand_args_for_die(
         times: Sequence[int]) -> Counts[T]:
 
     subdatas = [expand_arg(arg) for arg in args]
-    data = merge_weights_lcm(subdatas, times)
-    return Counts(data.items())
+    return Counts(merge_weights_lcm(subdatas, times).items())
 
 
 def expand_args_for_deck(
@@ -64,61 +61,51 @@ def expand_args_for_deck(
         times: Sequence[int]) -> Counts[T]:
 
     subdatas = [expand_arg(arg) for arg in args]
-    data = merge_duplicates(subdatas, times)
-    return Counts(data.items())
+    return Counts(merge_duplicates(subdatas, times).items())
 
 
 def expand_arg(
-        arg: T | icepool.Population[T] | icepool.RerollType) -> Mapping[T, int]:
+        arg: T | icepool.Population[T] | icepool.RerollType
+) -> Mapping[T, int]:
     if isinstance(arg, icepool.Population):
         return arg
     elif arg is icepool.Reroll:
         return {}
-    elif isinstance(arg, tuple):
-        if len(arg) == 0:
-            return {(): 1}  # type: ignore
-        sub_args: Iterable[Mapping[T, int]] = (
-            expand_arg_inside_tuple(sub_arg) for sub_arg in arg)
-        result: MutableMapping = defaultdict(int)
-        for t in itertools.product(*(sub_arg.items() for sub_arg in sub_args)):
-            outcomes, quantities = zip(*t)
-            final_quantity = math.prod(quantities)
-            result[outcomes] += final_quantity
-        return result
+    elif type(arg) == tuple:
+        arg = cast('T | icepool.Population[T]',
+                   icepool.cartesian_product(*arg, outcome_type=tuple))
+        if type(arg) == tuple:
+            return {arg: 1}  # type: ignore
+        else:  # converted to Population
+            return arg  # type: ignore
     else:
         return {arg: 1}
 
 
-def expand_arg_inside_tuple(
-        arg: T | icepool.Population[T] | icepool.RerollType) -> Mapping[T, int]:
-    if isinstance(arg, icepool.Again):
-        raise TypeError('Again is not allowed inside tuple outcomes.')
-    elif arg is icepool.Reroll:
-        raise TypeError('Reroll is not allowed inside tuple outcomes.')
-    else:
-        return expand_arg(arg)
-
-
 def merge_weights_lcm(subdatas: Sequence[Mapping[T, int]],
                       weights: Sequence[int]) -> Mapping[T, int]:
     """Merge for dice.
 
     Every subdata gets total weight proportional to the corresponding element of `weights`.
-
-    The final denominator is equal to the primary denominator times the LCM of the secondary denominators.
     """
     if any(x < 0 for x in weights):
         raise ValueError('weights cannot be negative.')
 
     subdata_denominators = [sum(subdata.values()) for subdata in subdatas]
 
-    denominator_lcm = math.lcm(*(d for d in subdata_denominators if d > 0))
+    denominator_lcm = math.lcm(*(d // math.gcd(d, w)
+                                 for d, w in zip(subdata_denominators, weights)
+                                 if d > 0 and w > 0))
 
     data: MutableMapping[Any, int] = defaultdict(int)
     for subdata, subdata_denominator, w in zip(subdatas, subdata_denominators,
                                                weights):
-        factor = denominator_lcm * w // subdata_denominator if subdata_denominator else 0
+        if subdata_denominator == 0 or w == 0:
+            continue
+        factor = denominator_lcm * w // subdata_denominator
         for outcome, weight in subdata.items():
+            if weight == 0:
+                continue
             data[outcome] += weight * factor
     return data
 
@@ -135,7 +122,11 @@ def merge_duplicates(subdatas: Sequence[Mapping[T, int]],
 
     data: MutableMapping[Any, int] = defaultdict(int)
     for subdata, subdup in zip(subdatas, duplicates):
+        if subdup == 0:
+            continue
         for outcome, dup in subdata.items():
+            if dup == 0:
+                continue
             data[outcome] += dup * subdup
 
     return data
