@@ -2,13 +2,11 @@
 
 __docformat__ = 'google'
 
-import icepool
 from icepool.evaluator.multiset_evaluator import MultisetEvaluator
 
 from abc import abstractmethod
 
-from icepool.typing import Order
-from typing import Any, Literal
+from typing import Any
 
 
 class ComparisonEvaluator(MultisetEvaluator[Any, bool]):
@@ -27,7 +25,7 @@ class ComparisonEvaluator(MultisetEvaluator[Any, bool]):
     def default_outcome() -> bool:
         """The final outcome if both left and right have no outcomes."""
 
-    def next_state(self, state, outcome, left, right):
+    def next_state(self, state, order, outcome, left, right):
         """Implementation."""
         has_any, has_all = state or (False, True)
         this_any, this_all = self.any_all(left, right)
@@ -35,16 +33,17 @@ class ComparisonEvaluator(MultisetEvaluator[Any, bool]):
         has_any = has_all and (has_any or this_any)
         return has_any, has_all
 
-    def final_outcome(self, final_state) -> bool:
+    def final_outcome(  # type: ignore
+            self, final_state, order, outcomes, left_size, right_size) -> bool:
         """Implementation."""
         if final_state is None:
             return self.default_outcome()
         has_any, has_all = final_state
         return has_any and has_all
 
-    def order(self) -> Literal[Order.Any]:
-        """Allows any order."""
-        return Order.Any
+    @property
+    def next_state_key(self):
+        return type(self)
 
 
 class IsProperSubsetEvaluator(ComparisonEvaluator):
@@ -110,7 +109,7 @@ class IsNotEqualSetEvaluator(ComparisonEvaluator):
 class IsDisjointSetEvaluator(ComparisonEvaluator):
 
     def any_all(self, left: int, right: int) -> tuple[bool, bool]:
-        return True, not ((left > 0) and right > 0)
+        return True, not (left > 0 and right > 0)
 
     @staticmethod
     def default_outcome() -> bool:
